""" Gandi CLI core modules.

Contains:
- XML-RPC connection handler
- Configuration handler
- Base class, which load modules and commands
- Custom command line validation parameters
- Output methods
"""
